@@ -1,18 +1,19 @@
-# Authors: Clemens Brunner <clemens.brunner@gmail.com>
+# © MNELAB developers
 #
 # License: BSD (3-clause)
 
+from collections import Counter, defaultdict
+from copy import deepcopy
+from functools import wraps
 from os.path import getsize, join, split, splitext
 from pathlib import Path
-from collections import Counter, defaultdict
-from functools import wraps
-from copy import deepcopy
 
-import numpy as np
 import mne
+import numpy as np
 
-from .utils import has_locations
 from .io import read_raw, write_raw
+from .io.readers import split_name_ext
+from .utils import count_locations
 
 
 class LabelsNotFoundError(Exception):
@@ -104,6 +105,7 @@ class Model:
     @data_changed
     def load(self, fname, *args, **kwargs):
         """Load data set from file."""
+        fname = str(Path(fname).resolve())
         data = read_raw(fname, *args, **kwargs, preload=True)
         argstr = ", " + f"{', '.join(f'{v}' for v in args)}" if args else ""
         if kwargs:
@@ -113,14 +115,30 @@ class Model:
         self.history.append(f'data = read_raw("{fname}"{argstr}{kwargstr}, preload=True)'.
                             replace("'", '"'))
         fsize = getsize(data.filenames[0]) / 1024**2  # convert to MB
-        name, ext = Path(fname).stem, "".join(Path(fname).suffixes)
-        self.insert_data(defaultdict(lambda: None, name=name, fname=fname,
-                                     ftype=ext.upper()[1:], fsize=fsize, data=data,
-                                     dtype="raw"))
+        name, ext = split_name_ext(fname)
+        self.insert_data(defaultdict(
+            lambda: None,
+            name=name,
+            fname=fname,
+            ftype=ext.upper()[1:],
+            fsize=fsize,
+            data=data,
+            dtype="raw",
+            montage=None,
+            events=np.empty((0, 3), dtype=int),
+            event_mapping=defaultdict(str),
+        ))
 
     @data_changed
-    def find_events(self, stim_channel, consecutive=True, initial_event=True,
-                    uint_cast=True, min_duration=0, shortest_event=0):
+    def find_events(
+        self,
+        stim_channel,
+        consecutive=True,
+        initial_event=True,
+        uint_cast=True,
+        min_duration=0,
+        shortest_event=0
+    ):
         """Find events in raw data."""
         events = mne.find_events(
             self.current["data"],
@@ -133,7 +151,20 @@ class Model:
         )
         if events.shape[0] > 0:  # if events were found
             self.current["events"] = events
-            self.history.append("events = mne.find_events(data)")
+            hist = "events = mne.find_events(data"
+            hist += f", stim_channel={stim_channel!r}"
+            if consecutive != "increasing":
+                hist += f", consecutive={consecutive!r}"
+            if initial_event:
+                hist += f", initial_event={initial_event!r}"
+            if uint_cast:
+                hist += f", uint_cast={uint_cast!r}"
+            if min_duration > 0:
+                hist += f", min_duration={min_duration!r}"
+            if shortest_event != 2:
+                hist += f", shortest_event={shortest_event!r}"
+            hist += ")"
+            self.history.append(hist)
 
     @data_changed
     def events_from_annotations(self):
@@ -217,20 +248,25 @@ class Model:
 
     @data_changed
     def import_events(self, fname):
-        """Import events from a CSV file."""
-        pos, desc = [], []
-        with open(fname) as f:
-            f.readline()  # skip header
-            for line in f:
-                p, d = [int(token.strip()) for token in line.split(",")]
-                pos.append(p)
-                desc.append(d)
-        events = np.column_stack((pos, desc))
-        events = np.insert(events, 1, 0, axis=1)  # insert zero column
-        if self.current["events"] is not None:
-            events = np.row_stack((self.current["events"], events))
-            events = np.unique(events, axis=0)
-        self.current["events"] = events
+        """Import events from a CSV or FIF file."""
+        if fname.lower().endswith(".csv"):
+            pos, desc = [], []
+            with open(fname) as f:
+                f.readline()  # skip header
+                for line in f:
+                    p, d = [int(token.strip()) for token in line.split(",")]
+                    pos.append(p)
+                    desc.append(d)
+            events = np.column_stack((pos, desc))
+            events = np.insert(events, 1, 0, axis=1)  # insert zero column
+            if self.current["events"] is not None:
+                events = np.row_stack((self.current["events"], events))
+                events = np.unique(events, axis=0)
+            self.current["events"] = events
+        elif fname.lower().endswith(".fif"):
+            self.current["events"] = mne.read_events(fname)
+        else:
+            raise ValueError(f"Unsupported event file: {fname}")
 
     @data_changed
     def import_annotations(self, fname):
@@ -274,11 +310,24 @@ class Model:
         dtype = self.current["dtype"].capitalize()
         reference = self.current["reference"]
         events = self.current["events"]
-        locations = has_locations(self.current["data"].info)
+        montage = self.current["montage"]
         ica = self.current["ica"]
 
-        length = f"{data.times[-1] - data.times[0]:.6g} s"
-        samples = f"{len(data.times)}"
+        fs = data.info["sfreq"]
+        n_samples = len(data.times)
+        samples = f"{n_samples:,}".replace(",", "\u2009")
+
+        seconds = n_samples / fs
+        minutes, seconds = divmod(seconds, 60)
+        hours, minutes = divmod(minutes, 60)
+        hours, minutes = int(hours), int(minutes)
+        if hours > 0:
+            length = f"{hours}\u2009h {minutes}\u2009m {seconds:.3g}\u2009s"
+        elif minutes > 0:
+            length = f"{minutes}\u2009m {seconds:.3g}\u2009s"
+        else:
+            length = f"{seconds:.3g}\u2009s"
+
         if self.current["dtype"] == "epochs":  # add epoch count
             length = f"{self.current['data'].events.shape[0]} x {length}"
             samples = f"{self.current['data'].events.shape[0]} x {samples}"
@@ -296,20 +345,32 @@ class Model:
         chans = ", ".join([" ".join([str(v), k.upper()]) for k, v in chans])
 
         if events is not None and events.shape[0] > 0:
-            nevents = events.shape[0]
-            unique = [str(e) for e in sorted(set(events[:, 2]))]
-            if len(unique) > 20:  # do not show all events
-                first = ", ".join(unique[:10])
-                last = ", ".join(unique[-10:])
-                events = f"{nevents} ({first + ', ..., ' + last})"
+            unique, counts = np.unique(events[:, 2], return_counts=True)
+            events = f"{events.shape[0]} ("
+            if len(unique) < 8:
+                events += ", ".join([f"{u}: {c}" for u, c in zip(unique, counts)])
+            elif 8 <= len(unique) <= 12:
+                events += ", ".join([f"{u}" for u in unique])
             else:
-                events = f"{nevents} ({', '.join(unique)})"
+                first = ", ".join([f"{u}" for u in unique[:6]])
+                last = ", ".join([f"{u}" for u in unique[-6:]])
+                events += f"{first}, ..., {last}"
+            events += ")"
         else:
             events = "-"
 
         if isinstance(reference, list):
             reference = ",".join(reference)
 
+        locations = count_locations(self.current["data"].info)
+
+        if montage is None and not locations:
+            montage_text = "none"
+        elif montage is None and locations:
+            montage_text = f"custom ({locations}/{data.info['nchan']} locations)"
+        elif montage:
+            montage_text = f"{montage} ({locations}/{data.info['nchan']} locations)"
+
         if ica is not None:
             method = ica.method.title()
             if method == "Fastica":
@@ -333,18 +394,19 @@ class Model:
                 "Size in memory": f"{data.get_data().nbytes / 1024**2:.2f} MB",
                 "Channels": f"{nchan} (" + chans + ")",
                 "Samples": samples,
-                "Sampling frequency": f"{data.info['sfreq']:.6g} Hz",
+                "Sampling frequency": f"{fs:.6g} Hz",
                 "Length": length,
                 "Events": events,
                 "Annotations": annots,
                 "Reference": reference if reference else "-",
-                "Locations": "Yes" if locations else "-",
+                "Montage": montage_text,
                 "ICA": ica}
 
     @data_changed
-    def drop_channels(self, drops):
-        self.current["data"] = self.current["data"].drop_channels(drops)
-        self.current["name"] += " (channels dropped)"
+    def pick_channels(self, picks):
+        self.current["data"] = self.current["data"].pick(picks)
+        self.current["name"] += " (channels picked)"
+        self.history.append(f"data.pick({picks})")
 
     @data_changed
     def set_channel_properties(self, bads=None, names=None, types=None):
@@ -359,9 +421,33 @@ class Model:
             self.history.append(f"data.set_channel_types({types})")
 
     @data_changed
-    def set_montage(self, montage):
-        self.current["data"].set_montage(montage)
-        self.history.append(f"data.set_montage('{montage}', raise_if_subset=False)")
+    def rename_channels(self, new_names):
+        old_names = self.current["data"].info["ch_names"]
+        mapping = {o: n for o, n in zip(old_names, new_names) if o != n}
+        if not mapping:
+            return
+        mne.rename_channels(self.current["data"].info, mapping)
+        self.history.append(f"mne.rename_channels(data.info, {mapping})")
+
+    @data_changed
+    def set_montage(
+        self,
+        montage,
+        match_case=False,
+        match_alias=False,
+        on_missing="raise",
+    ):
+        self.current["data"].set_montage(
+            montage=montage,
+            match_case=match_case,
+            match_alias=match_alias,
+            on_missing=on_missing,
+        )
+        self.current["montage"] = montage
+        if montage is None:
+            self.history.append("data.set_montage(None)")
+        else:
+            self.history.append(f"data.set_montage({montage!r}, match_case={match_case}, match_alias={match_alias}, on_missing={on_missing!r})")  # noqa: E501
 
     @data_changed
     def filter(self, low, high):
@@ -449,16 +535,22 @@ class Model:
         self.current["name"] += " (interpolated)"
 
     @data_changed
-    def epoch_data(self, events, tmin, tmax, baseline):
-        epochs = mne.Epochs(self.current["data"], self.current["events"], event_id=events,
+    def epoch_data(self, event_id, tmin, tmax, baseline):
+        epochs = mne.Epochs(self.current["data"], self.current["events"], event_id=event_id,
                             tmin=tmin, tmax=tmax, baseline=baseline, preload=True)
-        self.history.append(f"data = mne.Epochs(data, events, event_id={events}, "
+        self.history.append(f"data = mne.Epochs(data, events, event_id={event_id}, "
                             f"tmin={tmin}, tmax={tmax}, baseline={baseline}, preload=True)")
         self.current["data"] = epochs
         self.current["dtype"] = "epochs"
         self.current["events"] = self.current["data"].events
         self.current["name"] += " (epoched)"
 
+    @data_changed
+    def drop_bad_epochs(self, reject, flat):
+        self.current["data"].drop_bad(reject, flat)
+        self.current["name"] += " (dropped bad epochs)"
+        self.history.append(f"data.drop_bad({reject}, {flat})")
+
     @data_changed
     def convert_od(self):
         self.current["data"] = mne.preprocessing.nirs.optical_density(self.current["data"])
@@ -472,28 +564,21 @@ class Model:
         self.history.append("data = mne.preprocessing.nirs.beer_lambert_law(data)")
 
     @data_changed
-    def set_reference(self, ref):
+    def change_reference(self, add, ref):
+        self.current["reference"] = ref
+        if add:
+            mne.add_reference_channels(self.current["data"], add, copy=False)
+            self.history.append(f"mne.add_reference_channels(data, {add}, copy=False)")
+        if ref is None:
+            return
+
         self.current["reference"] = ref
         if ref == "average":
             self.current["name"] += " (average ref)"
-            self.current["data"].set_eeg_reference(ref)
-            self.history.append('data.set_eeg_reference("average")')
         else:
             self.current["name"] += " (" + ",".join(ref) + ")"
-            if set(ref) - set(self.current["data"].info["ch_names"]):
-                # add new reference channel(s) to data
-                try:
-                    mne.add_reference_channels(self.current["data"], ref, copy=False)
-                except RuntimeError:
-                    raise AddReferenceError("Cannot add reference channels to average "
-                                            "reference signals.")
-                else:
-                    self.history.append(f"mne.add_reference_channels(data, {ref}, "
-                                        f"copy=False)")
-            else:
-                # re-reference to existing channel(s)
-                self.current["data"].set_eeg_reference(ref)
-                self.history.append(f"data.set_eeg_reference({ref})")
+        self.current["data"].set_eeg_reference(ref)
+        self.history.append(f"data.set_eeg_reference({ref!r})")
 
     @data_changed
     def set_events(self, events):
@@ -502,3 +587,29 @@ class Model:
     @data_changed
     def set_annotations(self, onset, duration, description):
         self.current["data"].set_annotations(mne.Annotations(onset, duration, description))
+
+    @data_changed
+    def move_data(self, source, target):
+        """
+        Change the position of a single data set in `self.data`.
+
+        Parameters
+        ----------
+        source : int
+            The data set's initial index.
+        target : int
+            The index the data set should be moved to.
+        """
+        # first the data set is copied to the target index
+        self.data.insert(target, self.data[source])
+        self.history.append(f"datasets.insert({target}, datasets[{source}])")
+        # if moved to the front, the source index is increased by 1
+        if source > target:
+            source += 1
+        # if moved to the back, the new index (after removing the original
+        # data set) will be 1 less that the target index
+        else:
+            target -= 1
+        self.index = target
+        self.data.pop(source)
+        self.history.append(f"datasets.pop({source})")
