@@ -1,8 +1,8 @@
-# Authors: Clemens Brunner <clemens.brunner@gmail.com>
+# © MNELAB developers
 #
 # License: BSD (3-clause)
 
-from qtpy.QtWidgets import QWidget, QVBoxLayout, QGridLayout, QLabel, QSizePolicy
+from PySide6.QtWidgets import QGridLayout, QLabel, QSizePolicy, QVBoxLayout, QWidget
 
 
 class InfoWidget(QWidget):
